package com.baidu.netdisk.label.view.base

import android.content.Context
import android.graphics.drawable.GradientDrawable
import android.text.TextUtils
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import com.baidu.netdisk.R
import com.netdisk.themeskin.loader.SkinManager

/**
 * 标签制作
 *
 * @author wanglihong01
 */
class GradientTextView(private val mContext: Context) {

    private val mGradientDrawable: GradientDrawable
    private val mLabelTextView: TextView
    private val mCancelBtn: ImageView
    private val mLayout: RelativeLayout

    init {
        mGradientDrawable = GradientDrawable()
        mLayout = LayoutInflater.from(mContext).inflate(R.layout.character_text_layout,
                null, false) as RelativeLayout
        mLabelTextView = mLayout.findViewById(R.id.character_label)
        mCancelBtn = mLayout.findViewById(R.id.character_cancel_label)
//        mCancelBtn.background = SkinManager.getInstance().getDrawable(R.drawable.character_label_cancel)
        mCancelBtn.setImageDrawable(SkinManager.getInstance().getDrawable(R.drawable.character_label_cancel))
        initGradientDrawable()
        initLabelTextView()
    }

    /**
     * 初始化GradientDrawable，标签的背景色和圆弧
     */
    private fun initGradientDrawable() {
        mGradientDrawable.setColor(SkinManager.getInstance().getColor(R.color.bg_dn_common_color_light))
        mGradientDrawable.setCornerRadius(dip2px(10f).toFloat())
    }

    /**
     * 初始化标签
     */
    private fun initLabelTextView() {
        mLabelTextView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 12f)
        mLabelTextView.setPadding(dip2px(13f), 0, dip2px(13f), 0)
        mLabelTextView.setSingleLine(true)
    }

    /**
     * dp-->px
     *
     * @param dipValue
     * @return
     */
    private fun dip2px(dipValue: Float): Int {
        val scale = mContext.getResources().getDisplayMetrics().density
        return (dipValue * scale + 0.5f).toInt()
    }

    /**
     * 是否为空
     *
     * @param str
     * @return
     */
    private fun empty(str: Int): Boolean {
        return str == -1
    }

    /**
     * 标签字体颜色
     *
     * @param object
     * @return
     */
    fun setTextColor(color: Int): GradientTextView {
        if (color != 0) {
            try {
                mLabelTextView.setTextColor(color)
            } catch (e: Exception) {

            }

        }
        return this
    }

    /**
     * 标签背景颜色
     *
     * @param object
     * @return
     */
    fun setBackgroundColor(color: Int): GradientTextView {
        if (!empty(color)) {
            mGradientDrawable.setColor(color)
        }
        return this
    }

    /**
     * 标签外框颜色
     *
     * @param object
     * @return
     */
    fun setStrokeColor(color: Int): GradientTextView {
        if (!empty(color)) {
            mGradientDrawable.setStroke(dip2px(1f), color)
        }
        return this
    }

    fun setStrokeRadius(radius: Int): GradientTextView {
        mGradientDrawable.setCornerRadius(dip2px(radius.toFloat()).toFloat())
        return this
    }

    /**
     * 设置标签内容
     *
     * @param info
     * @return
     */
    fun setLabelText(info: String): GradientTextView {
        if (!TextUtils.isEmpty(info)) {
            mLabelTextView.setText(info)
        }
        return this
    }

    /**
     * 设置标签字体大小
     */
    fun setTextSize(size: Int): GradientTextView {
        mLabelTextView.setTextSize(TypedValue.COMPLEX_UNIT_SP, size.toFloat())
        return this
    }

    /**
     * 设置标签字体粗体
     */
    fun setTextBold(isBold: Boolean): GradientTextView {
        val paint = mLabelTextView.paint
        paint.isFakeBoldText = isBold
        return this
    }

    /**
     * 设置监听事件
     *
     * @param listener
     */
    fun setOnClickListener(listener: BaseLabelListView.IOnItemClickListener?, label: String,
                           position: Int): GradientTextView {
        mLabelTextView.setOnClickListener(object : View.OnClickListener {
            override fun onClick(v: View) {
                listener?.onClick(label, position)
            }
        })
        mCancelBtn.setOnClickListener(object : View.OnClickListener {
            override fun onClick(v: View) {
                listener?.onDeleteTag(label, position)
            }
        })
        return this
    }

    /**
     * 设置标签右上角x的可见性
     */
    fun setSelectMode(viewMode: Boolean): GradientTextView {
        if (viewMode) {
            mCancelBtn.visibility = View.GONE
        } else {
            mCancelBtn.visibility = View.VISIBLE
        }
        return this
    }

    /**
     * 构造TextView
     *
     * @return
     */
    fun build(): RelativeLayout {

        mLabelTextView.setBackgroundDrawable(mGradientDrawable)
        return mLayout
    }

}
