package com.baidu.netdisk.p2p

import android.app.Notification
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.support.v4.content.LocalBroadcastManager
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.util.NotificationUtil.ONGOING_DOWNLOAD
import com.baidu.netdisk.util.NotificationUtil.VIDEO_NOTIFICATION

/**
 * 下载绑定通知栏维护中心
 *
 * @author wangyang34
 * @since 2019年7月16日
 */
@Tag("P2PNotificationCenter")
class P2PNotificationCenter(val context: Context, private val p2pServerProxy: P2PServiceProxy?) {

    companion object {

        /** 视频播放状态的action */
        const val ACTION_P2P_VIDEO_PLAY_STATUS_CHANGE = "com.baidu.netdisk.p2p.video_play_status_change"

        /** 视频是否播放 */
        const val EXTRA_IS_VIDEO_PLAYING = "extra_is_video_playing"

        /** 有下载任务的action */
        const val ACTION_P2P_DOWNLOAD_ACTIVE = "com.baidu.netdisk.p2p.download_active"

        /** 下载任务都完成的action */
        const val ACTION_P2P_DOWNLOAD_ALL_COMPLETE = "com.baidu.netdisk.p2p.download_all_complete"

        /** 下载的通知栏 */
        const val EXTRA_DOWNLOAD_NOTIFICATION = "extra_download_notification"

        /** 下载对应的id */
        const val EXTRA_DOWNLOAD_ID = "extra_download_id"
    }

    /** 是否在播放视频 */
    private var mIsVideoPlaying = false

    /** 通知栏 */
    private var mVideoNotification: VideoNotification? = null

    /** 是否下载进行中 */
    private var mIsDownloading = false

    /** 下载通知栏id */
    private var mDownloadNotificationID = ONGOING_DOWNLOAD

    /** 下载通知栏 */
    private var mDownloadNotification: Notification? = null

    /**
     * 创建
     */
    fun onCreate() {
        "onCreate".d()
        mLocalReceiver?.let {
            val filter = IntentFilter()
            filter.addAction(ACTION_P2P_VIDEO_PLAY_STATUS_CHANGE)
            filter.addAction(ACTION_P2P_DOWNLOAD_ACTIVE)
            filter.addAction(ACTION_P2P_DOWNLOAD_ALL_COMPLETE)
            LocalBroadcastManager.getInstance(context).registerReceiver(it, filter)
        }
    }

    /**
     * 销毁
     */
    fun onDestroy() {
        "onDestroy".d()
        mLocalReceiver?.let {
            LocalBroadcastManager.getInstance(context).unregisterReceiver(it)
            mLocalReceiver = null
        }
    }

    /**
     * 设置视频播放状态
     * @param isPlaying 是否有视频播放
     */
    fun setIsVideoPlaying(isPlaying: Boolean) {
        "setIsVideoPlaying isPlaying:$isPlaying".d()
        if (mIsVideoPlaying == isPlaying) {
            return
        }
        mIsVideoPlaying = isPlaying
        if (mVideoNotification == null) {
            mVideoNotification = VideoNotification()
        }
        if (mIsVideoPlaying) {
            mVideoNotification?.show()
        } else {
            mVideoNotification?.close()
        }
        onStateChange()
    }

    /**
     *
     * 处理有下载任务的情况
     * @param notification 通知栏
     * @param id
     *
     */
    fun setIsDownloading(isDownloading: Boolean, notification : Notification?, id: Int) {
        if (mIsDownloading == isDownloading) {
            return
        }
        "setIsDownloading isDownloading: $isDownloading notification: $notification id: $id".d()
        if (mIsDownloading == isDownloading) {
            return
        }
        mIsDownloading = isDownloading
        mDownloadNotificationID = id
        mDownloadNotification = notification
        onStateChange()
    }

    /**
     * 使用p2p的状态改变了
     */
    private fun onStateChange() {
        "onStateChange".d()
        if (mIsVideoPlaying || mIsDownloading) {
            getNotification()?.let {
                p2pServerProxy?.bindNotification(it)
            }
            p2pServerProxy?.startProcess()
        } else {
            p2pServerProxy?.unbindNotification()
        }
    }

    /**
     * 获取可用的通知栏
     * @return 通知栏
     */
    private fun getNotification() : Pair<Int, Notification>? {
        if (mIsVideoPlaying) {
            mVideoNotification?.let {
                return Pair(VIDEO_NOTIFICATION, it.mNotification)
            }
        }
        if (mIsDownloading) {
            mDownloadNotification?.let {
                return Pair(mDownloadNotificationID, it)
            }
        }
        return null
    }

    /** 广播 */
    private var mLocalReceiver: BroadcastReceiver? = object : BroadcastReceiver() {

        override fun onReceive(context: Context, intent: Intent) {
            when(intent.action) {
                ACTION_P2P_VIDEO_PLAY_STATUS_CHANGE -> {
                    setIsVideoPlaying(intent.getBooleanExtra(EXTRA_IS_VIDEO_PLAYING, false))
                }
                ACTION_P2P_DOWNLOAD_ACTIVE -> {
                    var notification = intent.getParcelableExtra<Notification>(EXTRA_DOWNLOAD_NOTIFICATION)
                    var id = intent.getIntExtra(EXTRA_DOWNLOAD_ID, ONGOING_DOWNLOAD)
                    setIsDownloading(true, notification, id)
                }
                ACTION_P2P_DOWNLOAD_ALL_COMPLETE -> {
                    setIsDownloading(false, null, ONGOING_DOWNLOAD)
                }
            }
        }
    }
}