package com.baidu.netdisk.p2p

import android.app.Notification
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.os.IBinder
import com.baidu.library.Key
import com.baidu.library.P2P
import com.baidu.netdisk.kernel.util.TimeUtil
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.service.NetdiskService
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import java.util.concurrent.TimeUnit
import kotlin.math.pow

/**
 * 下载服务的代理类 做一些启动停止操作
 *
 * @author wangyang34
 * @since 2019年7月31日
 *
 * @param mContext 上下文
 * @param mNetdiskService 网盘服务
 */
@Tag("P2PServiceProxy")
class P2PServiceProxy(var mContext: Context?, val mNetdiskService: NetdiskService?) : ServiceConnection {

    /** 计算延时的时间段长度  */
    private val mTraceSpanTime: Long = 10 * TimeUtil.ONEMINUTE

    /** 异步任务句柄 */
    private var mDisposable: Disposable? = null

    /** p2p服务连接 */
    private var mServiceConnection: P2PServiceConnection? = null

    /** 启动时间 */
    private var mStartTrace: MutableList<Long> = mutableListOf()

    /** 通知栏是否可视 */
    private var mNotification: Pair<Int, Notification>? = null

    /**
     * 启动p2p远程服务
     */
    fun startProcess() {
        "startProcess".d()
        mStartTrace.clear()
        startProcessInner()
    }

    /**
     * 启动p2p远程服务
     */
    private fun startProcessInner() {
        if (mContext == null) {
            "mContext 为空".d()
            return
        }
        val delayTime = getDelayTime()
        "startProcessInner mStartTrace $mStartTrace delay $delayTime".d()
        mStartTrace.add(System.currentTimeMillis())
        mDisposable?.dispose()
        mDisposable = Observable.just(this)
                .delay(delayTime, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    if (mContext == null) {
                        return@subscribe
                    }
                    if (mServiceConnection == null) {
                        mServiceConnection = P2PServiceConnection(it)
                    }
                    val intent = Intent(mContext, NetdiskDownloadService::class.java)
                    var br = mContext!!.bindService(intent, mServiceConnection, Context.BIND_AUTO_CREATE)
                    "startProcessInner br: $br mContext: $mContext".d()
                    if (br) {
                        return@subscribe
                    }
                    NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                            NetdiskStatisticsLogForMutilFields.StatisticsKeys.P2P_BIND_DOWNLOAD_SERVICE_FAIL)
                }
    }

    /**
     * 停止p2p远程服务
     */
    fun stopProcess() {
        "stopProcess $mContext".d()
        mDisposable?.dispose()
        mDisposable = null
        if (mContext == null || mServiceConnection == null) {
            return
        }
        mContext?.unbindService(mServiceConnection)
        mServiceConnection = null
        mContext = null
    }

    override fun onServiceDisconnected(name: ComponentName?) {
        "onServiceDisconnected".d()
        startProcessInner()
    }

    override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
        "onServiceConnected".d()
        P2P.getInstance().getParameter(Key.HTTP_SERVER_PORT)
        mNetdiskService?.downloadTaskManager?.restartAllTasks()
        bindNotification(mNotification)
    }

    /**
     * 绑定通知栏
     * @param notification 通知栏
     */
    fun bindNotification(notification: Pair<Int, Notification>?) {
        "bindNotification $notification".d()
        mNotification = notification
        mNotification?.let {
            mServiceConnection?.bindNotification(it.second, it.first)
        }
    }

    /**
     * 设置通知栏是否可视
     * @param isVisibility 是否可视
     */
    fun unbindNotification() {
        "unbindNotification".d()
        mNotification = null
        mServiceConnection?.unbindNotification()
    }

    /**
     * 计算延时时间
     * @return 延时时间
     */
    private fun getDelayTime(): Long {
        val currentTime = System.currentTimeMillis()
        mStartTrace = mStartTrace.filter { (currentTime - it) < mTraceSpanTime } as MutableList<Long>
        val seed = Math.max(mStartTrace.size - 5, 0)
        return (seed * 2L).toDouble().pow(2.0).toLong()
    }
}