package com.baidu.netdisk.servicechannel

import android.annotation.SuppressLint
import android.support.annotation.IntDef
import android.support.v4.util.ArrayMap
import com.baidu.graph.sdk.utils.StreamUtils
import com.baidu.netdisk.NetDiskApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.base.utils.PersonalConfigKey.SERVICE_CHANNEL_CONFIG
import com.baidu.netdisk.kernel.android.util.file.CloseUtils
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.e
import com.baidu.netdisk.kotlin.extension.toArrayList
import com.baidu.netdisk.servicechannel.ServiceChannelConstants.MORE
import com.baidu.netdisk.servicechannel.entity.Channel
import com.baidu.netdisk.uiframe.card.CardStoreCardID
import com.baidu.netdisk.uiframe.card.CommonCardInfo
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import java.io.IOException
import java.io.InputStream
import java.lang.annotation.Retention
import java.lang.annotation.RetentionPolicy

/**
 * 10.0.30 版本对比
 *
 * @return true表示大于等于传入的版本，false表示小于
 */
inline fun String.versionCompare(versionRemote: String?): Boolean = try {
    versionRemote?.apply {
        var version = split(".")
        this@versionCompare.split(".").forEachIndexed { index, s ->
            if (s.toInt() < version[index].toInt()) {
                return@versionCompare false
            }
        }
    }
    true
} catch (e: NumberFormatException) {
    true
}

@Tag("ServiceChannelConfig")
object ServiceChannelConfig {
    private const val SERVICE_CHANNEL_DEFAULT = "service_channel.json"

    // 本地不存在该icon资源
    const val ICON_RES_NOT_EXIST = -1

    const val TYPE_COMMON_TITLE = CardStoreCardID.CARD_ITEM_TYPE_SERVICE_CHANNEL_TITLE // 常用项标题
    const val TYPE_COMMON_ITEM = CardStoreCardID.CARD_ITEM_TYPE_SERVICE_CHANNEL_ITEM  // 常用项item
    const val TYPE_OTHER_TITLE = CardStoreCardID.CARD_ITEM_TYPE_OTHER_SERVICE_CHANNEL_TITLE    // 其它项标题
    const val TYPE_OTHER_ITEM = CardStoreCardID.CARD_ITEM_TYPE_OTHER_SERVICE_CHANNEL_ITEM     // 其它项item

    @IntDef(TYPE_COMMON_TITLE.toLong(), TYPE_COMMON_ITEM.toLong(), TYPE_OTHER_TITLE.toLong(), TYPE_OTHER_ITEM.toLong())
    @Retention(RetentionPolicy.SOURCE)
    internal annotation class ChannelItemType

    // 当前配置信息的版本号
    @SuppressLint("NEW_STATIC_VARIABLE")
    var currentServiceChannelVersion: String = "10.0.30"

    // 图片映射
    private val imageIdMap = object : ArrayMap<String, Int>() {
        init {
            // 常用服务
            put(ServiceChannelConstants.ALBUM, R.drawable.service_album_icon)
            put(ServiceChannelConstants.VIDEO, R.drawable.service_video_icon)
            put(ServiceChannelConstants.NOTE, R.drawable.service_note_icon)
            put(ServiceChannelConstants.WECHAT, R.drawable.service_wechat_icon)
            put(ServiceChannelConstants.AUDIO, R.drawable.service_audio_icon)
            put(ServiceChannelConstants.MORE, R.drawable.service_more_icon)

            // 全部服务
            put(ServiceChannelConstants.DOCUMENT_MANAGE, R.drawable.service_file_manager_icon)
            put(ServiceChannelConstants.RESOURCE_BUY, R.drawable.service_file_insurance_icon)
            put(ServiceChannelConstants.DOCUMENT, R.drawable.service_document_icon)
            put(ServiceChannelConstants.APPLICATION, R.drawable.service_application_icon)
            put(ServiceChannelConstants.MEITUXIUXIU, R.drawable.service_item_pic_edit)
            put(ServiceChannelConstants.SEED, R.drawable.service_seed_icon)
            put(ServiceChannelConstants.KINGSOFT_DOCUMENT, R.drawable.service_document_eidt_icon)
            put(ServiceChannelConstants.NOVEL, R.drawable.service_novel_icon)
            put(ServiceChannelConstants.PRINT_SHOP, R.drawable.print_shop)
            put(ServiceChannelConstants.LABORATORY, R.drawable.netdisk_laboratory)
        }
    }

    // 常用服务不可以操作白名单
    @SuppressLint("NEW_STATIC_VARIABLE")
    var serviceItemWhiteList: List<String> = object : ArrayList<String>() {
        init {
            add(ServiceChannelConstants.ALBUM)
            add(ServiceChannelConstants.VIDEO)
            add(ServiceChannelConstants.MORE) // more与全部服务页虚线框复用
        }
    }

    /**
     * 获取资源ID
     *
     * @param channelCode item channel code
     * @return 资源id
     */
    fun getImageResId(channelCode: String): Int = imageIdMap[channelCode]?.apply { this }
            ?: ICON_RES_NOT_EXIST

    /**
     * string转ConfigServiceChannel
     */
    private fun jsonToConfigServiceChannel(data: String?) =
            Gson().fromJson<ConfigServiceChannel>(data, object : TypeToken<ConfigServiceChannel>() {}.type)

    /**
     * 载入assets中文件
     *
     * @param context context
     * @param path    文件路径
     * @return 文件中字符串
     */
    private fun loadAssetsFile(path: String): String? {
        var inputStream: InputStream? = null

        return try {
            inputStream = NetDiskApplication.getNetdiskApplication().applicationContext.assets.open(path)
            // 获取预置文件中的字符串
            StreamUtils.streamToString(inputStream)
        } catch (e: IOException) {
            "loadAssetsFile exception".e()
            null
        } finally {
            inputStream?.apply {
                CloseUtils.closeIO(this)
            }
        }
    }

    /**
     * 获取配置信息 并转成List<Channel>
     *
     * @return 返回过滤后的channel list
    </Channel> */
    private fun getChannelList(): List<Channel> = ConfigServiceChannel(ServerConfigKey
            .getCfgConfigBodyByType(ServerConfigKey.ConfigType.SERVICE_CHANNEL_CONFIG)).let { mispanConfig ->
        val localConfigServiceData = PersonalConfig.getInstance().getString(SERVICE_CHANNEL_CONFIG, null)
        // 本地mmkv数据是空，则直接读取mispan数据，mispan数据也是空，则读取asset目录数据
        if (localConfigServiceData.isNullOrEmpty()) {
            if (!mispanConfig.list.isNullOrEmpty() && !mispanConfig.versionCode.isNullOrEmpty()) {
                currentServiceChannelVersion = mispanConfig.versionCode
                mispanConfig.list.asSequence().filter { it != null }.toArrayList()
            } else {
                jsonToConfigServiceChannel(loadAssetsFile(SERVICE_CHANNEL_DEFAULT)).let {
                    currentServiceChannelVersion = it.versionCode
                    it.list
                }
            }
        } else {
            // 本地用户修改的数据和mispan数据都存在，进行diff合并
            mergeCongifList(mispanConfig, jsonToConfigServiceChannel(localConfigServiceData))
        }
    }

    /**
     * 合并mispan和用户本地修改的数据,合并规则如下：
     * 1：更新本地所有item的名称，跳转链接，icon下载链接
     * 2：去除mispan下架但本地存在的item
     * 3: 本地插入mispan新增item,
     *     插入原则： a: 若常用服务未满，则把新增item添加到常用服务满为止（添加到最后，不能修改用户已有item的排序）
     *              b: 若常用服务已满，则把新增item添加到其它服务（添加到最后，不能修改用户已有item的排序）
     */
    @SuppressLint("NEW_STATIC_VARIABLE")
    private var mergeCongifList: (mispanConfig: ConfigServiceChannel, localConfig: ConfigServiceChannel) -> List<Channel>
            = { mispanConfig, localConfig ->
        // mispan数据拉取失败或本地用户修改数据版本大于mispan时，使用本地数据
        if (mispanConfig.list.isNullOrEmpty() ||
                localConfig.versionCode.versionCompare(mispanConfig.versionCode)) {
            currentServiceChannelVersion = localConfig.versionCode
            localConfig.list
        } else {
            currentServiceChannelVersion = mispanConfig.versionCode
            var channelList = localConfig.list.asSequence()
                    .filter {
                        // 下架mispan不存在的item
                        mispanConfig.list.any { item -> item?.channelCode == it.channelCode }
                    }
                    .toArrayList()
                    .apply {
                        forEach {
                            // item修改，标题，链接等
                            mispanConfig.list.asSequence()
                                    .filter { item -> it.channelCode == item?.channelCode }
                                    .toArrayList()
                                    .forEach { item ->
                                        it.deeplink = item.deeplink
                                        it.extras = item.extras
                                        it.iconUrl = item.iconUrl
                                        it.title = item.title
                                    }
                        }
                    }
            mispanConfig.list.asSequence()
                    .filter {
                        // 过滤获取mispan新增的item
                        it != null && !(channelList.any { item -> item?.channelCode == it?.channelCode })
                    }
                    .toArrayList()?.apply {
                        forEach { it ->
                            // 如果常用服务没有满，新增加的常用服务item添加到常用服务中
                            if (it.itemType == TYPE_COMMON_ITEM && channelList.count { it.itemType == TYPE_COMMON_ITEM } < 6) {
                                channelList.add(channelList.indexOfLast { it.channelCode == MORE }, it)
                            } else {
                                it.itemType = TYPE_OTHER_ITEM
                                // 其余的新增item都加到全部服务中
                                channelList.add(it)
                            }
                        }
                    }
            channelList
        }
    }

    /**
     * 获取itemTypes类型中的数据
     *x
     * @param context
     * @param itemTypes 过滤的类型列表
     * @return 返回组装后 [com.baidu.netdisk.uiframe.containerimpl.list.ListDataInfo]内的data
     */
    fun getServiceChannelConfigList(@ServiceChannelConfig.ChannelItemType vararg itemTypes: Int): List<CommonCardInfo> =
            getChannelList().asSequence().filter {
                itemTypes.contains(it.itemType) && (it.channelCode != MORE || itemTypes.size == 1)
            }.map {
                var type = if (itemTypes.size == 1)
                    CardStoreCardID.CARD_ITEM_TYPE_COMMON_SERVICE_CHANNEL_ITEM
                else {
                    if (serviceItemWhiteList.contains(it.channelCode)) {
                        CardStoreCardID.CARD_ITEM_TYPE_SERVICE_CHANNEL_FIXED_ITEM
                    } else it.itemType
                }
                CommonCardInfo(type, it)
            }.toArrayList()
}