package com.baidu.netdisk.ui.cloudp2p.search

import android.content.Context
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.os.ResultReceiver
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.text.format.DateUtils.SECOND_IN_MILLIS
import android.view.KeyEvent
import android.view.LayoutInflater
import android.view.View
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import android.widget.ImageView
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.network.FallbackManager.HTTPS
import com.baidu.netdisk.base.network.ServerURL.PAN_DOMAIN
import com.baidu.netdisk.base.service.BaseServiceHelper
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.ServiceTypes
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.container.ContainerID.LABEL_SEARCH_RESULT_LIST_ID
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListContainer
import com.baidu.netdisk.uiframe.containerimpl.list.CommonRequestInfo
import com.baidu.netdisk.uiframe.containerimpl.list.ListStateContainer.LIST_STATE_IDLE
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.CommonListAdapter
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.MemoryListAdapterProxy
import com.baidu.netdisk.uiframe.containerimpl.sug.SugResponse
import com.baidu.netdisk.uiframe.containerimpl.sug.SugService
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView

/**
 * 标签搜索框
 *
 * @author wangyang34
 * @since 2019/11/6
 */
@Tag("LabelSearchContainer")
class LabelSearchContainer : BaseContainer() {

    companion object {

        /** tag */
        const val TAG = "LabelSearchContainer"

        /** 默认每次列表请求的条目 */
        const val REQUEST_LIST_COUNT = 5

        /** sug请求延迟时间 */
        const val REQUEST_SUG_DELAY: Long = SECOND_IN_MILLIS
    }

    /** 搜索框  */
    private lateinit var searchText: EditText

    /** 清除按钮  */
    private lateinit var cleanBtn: ImageView

    /** 起始值  */
    private var mStartIndex = 0

    /** 搜索结果返回值  */
    private var mResultReceiver: SugResultReceiver = SugResultReceiver(this, Handler(), null)

    /** handle  */
    private var mHandler: Handler = Handler()

    override fun onCreateView(): View {
        val inflater = LayoutInflater.from(activity)
        var view = inflater.inflate(R.layout.label_search_frame, null)
        searchText = view.findViewById(R.id.search_text)
        cleanBtn = view.findViewById(R.id.clean_button)
        val query = mInfo.data as String
        if (!TextUtils.isEmpty(query)) {
            setSearchText(query)
        }
        searchText.addTextChangedListener(mSearchTextWatcher)
        searchText.setOnEditorActionListener(mOnEditorActionListener)
        cleanBtn.setOnClickListener {
            NetDiskLog.d(TAG, "点击清除按钮")
            searchText.text.clear()
            postHideSug()
            resetSearchResult()
            postShowRecommend()
        }
        view.findViewById<View>(R.id.search_cancle).setOnClickListener {
            // 取消按钮点击直接返回
            activity.finish()
        }
        return view
    }

    override fun onAfterInitData() {
        super.onAfterInitData()
        val query = mInfo.data as String
        if (TextUtils.isEmpty(query)) {
            return
        }
        mStartIndex = 0
        onSearchLabel(query)
    }

    override fun onDestroyView() {
        canelSug()
    }

    override fun onCommonEvent(ev: CommonEvent): Boolean {
        when (ev.what) {
            MessageIDs.MESSAGE_SUG_ITEM_CLICK,
            MessageIDs.MESSAGE_RECOMMEND_LABEL_ITEM_CLICK -> {
                val query = ev.obj as String
                searchText.removeTextChangedListener(mSearchTextWatcher)
                setSearchText(query)
                searchText.addTextChangedListener(mSearchTextWatcher)
                mStartIndex = 0
                onSearchLabel(query)
            }
            MessageIDs.MESSAGE_COMMON_LIST_LOAD_MORE -> {
                val query = searchText.text.toString()
                onSearchLabel(query)
            }
            MessageIDs.MESSAGE_COMMON_LIST_LOAD_RETRY -> {
                val query = searchText.text.toString()
                mStartIndex -= REQUEST_LIST_COUNT
                onSearchLabel(query)
            }
        }
        return false
    }

    /**
     * 设置搜索框文字内容
     *
     * @param text 检索词
     */
    private fun setSearchText(text: String) {
        searchText.setText(text)
        searchText.setSelection(text.length);
        if (TextUtils.isEmpty(text)) {
            cleanBtn.visibility = View.GONE
            postShowRecommend()
        } else {
            cleanBtn.visibility = View.VISIBLE
            postHideRecommend()
        }
    }

    /**
     * 发起搜索
     *
     * @param query 检索词
     */
    private fun onSearchLabel(query: String) {
        canelSug()
        postHideSug()
        hideKeyBoard()
        postHideRecommend()
        postSearch(query)
    }

    /**
     * 重置搜索结果
     */
    private fun resetSearchResult() {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_STATE_CHANGE
        event.arg1 = LIST_STATE_IDLE
        postEventToPage(event)
        var listContainer = fragment.findContainerById(LABEL_SEARCH_RESULT_LIST_ID) as CommonListContainer
        val adapter = listContainer.recyclerView?.adapter as CommonListAdapter
        val adapterProxy = adapter.adapterProxy as MemoryListAdapterProxy
        adapterProxy?.clear()
    }

    /**
     * 发送搜索消息
     *
     * @param text 搜索的标签
     */
    private fun postSearch(text: String) {
        if (TextUtils.isEmpty(text)) {
            return
        }
        "postSearch $text".d()
        val params = ArrayList<Pair<String, String>>()
        val url = Uri.Builder()
                .scheme(HTTPS)
                .encodedAuthority(PAN_DOMAIN)
                .appendPath("ndrelation")
                .appendPath("tag")
                .appendPath("search")
                .build().toString()
            params.add(Pair("query", text))
            params.add(Pair("start", "$mStartIndex"))
            params.add(Pair("limit", "$REQUEST_LIST_COUNT"))
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_COMMON_LIST_REQUEST
        event.obj = CommonRequestInfo((mStartIndex == 0), url, true, params)
        postEventToPage(event)
        mStartIndex += REQUEST_LIST_COUNT
    }

    /**
     * 展示sug
     *
     * @param sugList sug列表
     */
    private fun postShowSug(sugList: ArrayList<String>) {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_SHOW_SUG
        event.obj = sugList
        postEventToPage(event)
    }

    /**
     * 隐藏sug
     *
     */
    private fun postHideSug() {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_HIDE_SUG
        postEventToPage(event)
    }

    /**
     * 隐藏推荐标签布局
     *
     */
    private fun postHideRecommend() {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_HIDE_RECOMMEND_LABEL
        postEventToPage(event)
    }

    /**
     * 显示推荐标签布局
     *
     */
    private fun postShowRecommend() {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_SHOW_RECOMMEND_LABEL
        postEventToPage(event)
    }

    /**
     * 隐藏键盘
     */
    private fun hideKeyBoard() {
        val imm = activity.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.hideSoftInputFromWindow(searchText.windowToken, 0)
    }

    /**
     * 文本编辑变化监听
     */
    private val mSearchTextWatcher = object : TextWatcher {

        override fun afterTextChanged(editable: Editable) {}

        override fun beforeTextChanged(str: CharSequence, start: Int, arg2: Int, arg3: Int) {}

        override fun onTextChanged(str: CharSequence, arg1: Int, arg2: Int, arg3: Int) {
            if (!TextUtils.isEmpty(str)) {
                // 搜索框文字变化，且有文字，则请求一次sug
                NetDiskLog.d(TAG, "搜索框文字变化，且有文字，则请求一次sug。文字是：" + str.toString())
                cleanBtn.visibility = View.VISIBLE
                postHideRecommend()
                canelSug()
                mHandler.postDelayed(Runnable {
                    getSug(context, mResultReceiver, str.toString())
                }, REQUEST_SUG_DELAY)

            } else {
                // 删掉所有文字要清空sug
                canelSug()
                postHideSug()
                postShowRecommend()
                cleanBtn.visibility = View.GONE
            }
        }
    }

    /**
     * 键盘按键监听
     */
    private val mOnEditorActionListener = object : TextView.OnEditorActionListener {
        override fun onEditorAction(v: TextView?, actionId: Int, event: KeyEvent?): Boolean {
            if (actionId == EditorInfo.IME_ACTION_SEARCH) { //判断动作标识是否搜索
                "mOnEditorActionListener ${v?.text.toString()}".d()
                mStartIndex = 0
                onSearchLabel(v?.text.toString())
                return true
            }
            return false
        }
    }

    /**
     * 取消sug请求
     *
     * @param context 上下文
     * @param resultReceiver 用于返回结果
     * @param inputText 输入的文本
     */
    fun canelSug() {
        mHandler.removeCallbacksAndMessages(null)
        if (!BaseServiceHelper.isNetWorkAvailable(context, null)) {
            return
        }
        val bduss = AccountUtils.getInstance().bduss
        if (TextUtils.isEmpty(bduss)) {
            return
        }
        val uid = AccountUtils.getInstance().uid
        if (TextUtils.isEmpty(uid)) {
            return
        }
        BaseServiceHelper.startTargetVersionService(context, BaseServiceHelper.buildIntent(context, bduss, uid, null)
                .putExtra(BaseExtras.EXTRA_SERVICE_TYPE, ServiceTypes.SUG)
                .setAction(SugService.ACTION_CANCEL_REQUEST_SUG))
    }

    /**
     * 获取sug列表
     *
     * @param context 上下文
     * @param resultReceiver 用于返回结果
     * @param inputText 输入的文本
     */
    fun getSug(context: Context, resultReceiver: ResultReceiver, inputText: String) {
        if (!BaseServiceHelper.isNetWorkAvailable(context, resultReceiver)) {
            return
        }
        val bduss = AccountUtils.getInstance().bduss
        if (TextUtils.isEmpty(bduss)) {
            return
        }
        val uid = AccountUtils.getInstance().uid
        if (TextUtils.isEmpty(uid)) {
            return
        }
        BaseServiceHelper.startTargetVersionService(context, BaseServiceHelper.buildIntent(context, bduss, uid, resultReceiver)
                .putExtra(BaseExtras.EXTRA_SERVICE_TYPE, ServiceTypes.SUG)
                .setAction(SugService.ACTION_REQUEST_SUG).putExtra(SugService.EXTRA_INPUT_TEXT, inputText))
    }

    /**
     * 接收到数据
     * @param response 网络返回数据
     */
    private fun onDataReceive(response: SugResponse) {
        NetDiskLog.d(TAG, "拿到接口返回数据，size：" + response?.mSugList?.size ?: "空")
        response?.mSugList?.apply {
            if (this.size <= 0) {
                postHideSug()
                return
            }
            postShowSug(this)
        }
    }

    /**
     * 列表结果接收器
     */
    private class SugResultReceiver internal constructor(reference: LabelSearchContainer,
                                                          handler: Handler, view: ResultView?)
        : BaseResultReceiver<LabelSearchContainer>(reference, handler, view) {

        override fun onInterceptResult(reference: LabelSearchContainer, resultCode: Int,
                                       resultData: Bundle?): Boolean {
            return if (reference.activity == null || reference.activity.isFinishing) {
                !super.onInterceptResult(reference, resultCode, resultData)
            } else {
                super.onInterceptResult(reference, resultCode, resultData)
            }
        }

        override fun onSuccess(reference: LabelSearchContainer, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            if (reference == null || resultData == null) {
                return
            }
            val response = resultData.get(BaseExtras.RESULT) as SugResponse
            reference.onDataReceive(response)
        }

        override fun onFailed(reference: LabelSearchContainer, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            if (resultData.getBoolean(BaseExtras.ERROR_NETWORK)) {
                ToastHelper.showToast(R.string.network_error)
            }
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

}