package com.baidu.netdisk.ui.xpan.extension.model

import android.os.Bundle
import com.baidu.netdisk.util.receiver.ErrorType
import java.io.Serializable

class CustomResult<out T>(val value: Any?) : Serializable {

    /**
     * Returns `true` if this instance represents successful outcome.
     * In this case [isFailure] returns `false`.
     */
    public val isSuccess: Boolean get() = value !is CustomFailure

    /**
     * Returns `true` if this instance represents failed outcome.
     * In this case [isSuccess] returns `false`.
     */
    public val isFailure: Boolean get() = value is CustomFailure

    // value & exception retrieval

    /**
     * Returns the encapsulated value if this instance represents [success][Result.isSuccess] or `null`
     * if it is [failure][Result.isFailure].
     *
     * This function is shorthand for `getOrElse { null }` (see [getOrElse]) or
     * `fold(onSuccess = { it }, onFailure = { null })` (see [fold]).
     */
    public inline fun getOrNull(): T? =
            when {
                isFailure -> null
                else -> value as? T
            }

    fun getFailureOrNull(): CustomFailure? = when {
        isSuccess -> null
        else -> value as? CustomFailure
    }

    /**
     * Returns a string `Success(v)` if this instance represents [success][Result.isSuccess]
     * where `v` is a string representation of the value or a string `Failure(x)` if
     * it is [failure][isFailure] where `x` is a string representation of the exception.
     */
    public override fun toString(): String =
            when (value) {
                is CustomFailure -> value.toString() // "Failure($exception)"
                else -> "Success($value)"
            }

    // companion with constructors

    /**
     * Companion object for [Result] class that contains its constructor functions
     * [success] and [failure].
     */
    companion object {
        /**
         * Returns an instance that encapsulates the given [value] as successful value.
         */
        inline fun <T> success(value: T): CustomResult<T> = CustomResult(value)

        /**
         * Returns an instance that encapsulates the given [exception] as failure.
         */
        inline fun <T> failure(errorType: ErrorType?, errorCode: Int, opCode: Int = -1, data: Bundle? = null): CustomResult<T> = CustomResult(createFailure(errorType, errorCode, opCode, data))
    }

    class CustomFailure(
            @JvmField
            val errorType: ErrorType?,
            @JvmField
            val errorCode: Int,
            @JvmField
            val opCode: Int = -1,
            @JvmField
            val data: Bundle?
    ) : Serializable {
        override fun equals(other: Any?): Boolean = other is CustomFailure && errorType == other.errorType
                && errorCode == other.errorCode && opCode == other.opCode

        override fun hashCode(): Int = errorType?.hashCode() ?: 0 + errorCode + opCode
        override fun toString(): String = "Failure(${errorType?.name})"
    }
}

fun createFailure(errorType: ErrorType?, errorCode: Int, opCode: Int, data: Bundle?): CustomResult.CustomFailure =
        CustomResult.CustomFailure(errorType, errorCode, opCode, data)