package com.baidu.netdisk.ui.xpan.transmission.share

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Context
import android.os.Bundle
import android.support.design.widget.BottomSheetDialog
import android.support.design.widget.BottomSheetDialogFragment
import android.support.v4.app.DialogFragment
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.kernel.android.util.TextTools
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.xpan.transmission.TransListRecordActivity
import com.baidu.netdisk.ui.xpan.transmission.adapter.NumericStepWheelAdapter
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.widget.wheel.WheelView
import com.baidu.netdisk.xpan.config.TransAssistantConfigKey
import com.baidu.netdisk.xpan.io.ErrorCode
import com.baidu.netdisk.xpan.io.parser.transmission.model.CreateTransTaskBean
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransTaskResponse
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys
import kotlinx.android.synthetic.main.layout_trans_share_panel.*

/**
 * 传输助手-分享面板
 *
 * @author rendawei
 * @since 2019/11/26
 */
class TransShareBottomSheet : BottomSheetDialogFragment() {

    companion object {
        const val TAG = "TransShareBottomSheet"

        private const val EXTRA_CREATE = "extra_create"
        private const val EXTRA_TASK = "extra_task"
        private const val EXTRA_EDITABLE = "extra_editable"
        private const val EXTRA_SHOW_BOTTOM = "extra_show_bottom"

        fun getTransShareDialog(createTransTaskBean: CreateTransTaskBean? = null, editable: Boolean,
                                transTaskResponse: TransTaskResponse? = null, showBottomBar: Boolean = true): TransShareBottomSheet {
            return TransShareBottomSheet().apply {
                arguments = Bundle().apply {
                    putParcelable(EXTRA_CREATE, createTransTaskBean)
                    putParcelable(EXTRA_TASK, transTaskResponse)
                    putBoolean(EXTRA_EDITABLE, editable)
                    putBoolean(EXTRA_SHOW_BOTTOM, showBottomBar)
                }
            }
        }
    }

    private var mViewModel: TransShareViewModel? = null
    private var mTransTaskResponse: TransTaskResponse? = null
    private var mEditEnable: Boolean = true
    private var mShowBottomBar: Boolean = true
    var mShareListener: ((CreateTransTaskBean, Boolean, TransTaskResponse?) -> Unit)? = null

    private var mInputMethodManager: InputMethodManager? = null
    private var mCreateParam: CreateTransTaskBean = CreateTransTaskBean(CreateTransTaskBean.CREATE_TYPE_SHARE).apply {
        title = TransShareViewModel.getDefaultTaskName()
        // 默认七天
        period = PersonalConfig.getInstance().getInt(getConfigKey(TransAssistantConfigKey.TRANS_SHARE_EXPIRED_SUFFIX), 7)
        // 默认60人
        uidLimit = PersonalConfig.getInstance().getInt(getConfigKey(TransAssistantConfigKey.TRANS_SHARE_UID_SUFFIX), 60)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        setStyle(DialogFragment.STYLE_NO_TITLE, R.style.NetDisk_BottomSheetTrans)
        super.onCreate(savedInstanceState)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.layout_trans_share_panel, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initParam()
        initView()
        initItemListener()
        initData()
    }

    override fun onDestroyView() {
        et_name?.let {
            hideInput(it)
        }
        activity?.window?.decorView?.let {
            NetDiskLog.d(TAG, "removeOnGlobalLayoutListener")
            SoftKeyboardHelper.getInstance().removeOnGlobalLayoutListener(it)
        }
        SoftKeyboardHelper.release()
        super.onDestroyView()
    }

    private fun initParam() {
        arguments?.apply {
            (getParcelable(EXTRA_CREATE) as? CreateTransTaskBean)?.also {
                mCreateParam = it
            }
            (getParcelable(EXTRA_TASK) as? TransTaskResponse)?.also {
                mTransTaskResponse = it
                // 显示上次分享的标题
                it.title?.takeIf { t ->
                    t.isNotEmpty()
                }?.let { lastTitle ->
                    mCreateParam.title = lastTitle
                }
            }
            mEditEnable = getBoolean(EXTRA_EDITABLE, true)
            mShowBottomBar = getBoolean(EXTRA_SHOW_BOTTOM, true)
        }
    }

    private fun initView() {
        mInputMethodManager = mInputMethodManager
                ?: (activity?.getSystemService(Context.INPUT_METHOD_SERVICE) as? InputMethodManager)
        et_name.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {
                s?.let {
                    val text = s.toString()
                    if (TextUtils.isEmpty(text.trim())) {
                        s.clear()
                    } else {
                        val fetchCharNumber = TextTools.fetchCharNumber(text)
                        NetDiskLog.d(TAG, "fetchCharNumber:$fetchCharNumber")
                        val diffLength = fetchCharNumber - TransShareViewModel.NICKNAME_MAX_LENGTH
                        if (diffLength > 4) {
                            s.delete(s.length - diffLength / 2, s.length)
                        } else if (diffLength > 0) {// 递归
                            s.delete(s.length - 1, s.length)
                        }
                    }
                    mCreateParam.title = s.toString()
                }
            }

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
            }

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
            }
        })
        val decorView = activity!!.window.decorView
        SoftKeyboardHelper.getInstance().setOnSoftGlobalLayoutChangeListener(decorView, "et_name",
                object : OnSoftGlobalLayoutChangeListener {
                    override fun keyBoardShow(id: String?, height: Int) {
                        NetDiskLog.d(TAG, "keyBoardShow")
                        et_name?.isCursorVisible = true
                    }

                    override fun keyBoardHide(id: String?, height: Int) {
                        NetDiskLog.d(TAG, "keyBoardHide")
                        et_name?.isCursorVisible = false
                    }

                    override fun onGlobalLayout(id: String?) {
                    }
                })
        et_name.setOnEditorActionListener { v, actionId, _ ->
            when (actionId) {
                EditorInfo.IME_ACTION_DONE -> {
                    (v as? EditText)?.let {
                        hideInput(it)
                    }
                    true
                }
                else -> false
            }
        }
        et_name.setOnClickListener {
            et_name.isCursorVisible = true
        }
        iv_icon.setOnClickListener {
            et_name.isCursorVisible = true
            setEditable(mEditEnable)
            showInput(et_name)
            et_name?.requestFocus()
        }
        val notice = TextTools.highlightText(getString(R.string.trans_assistant_share_to_see),
                resources.getColor(R.color.trans_text_color_06A7FF), "去查看")
        tv_share_finish.text = notice
        rl_wheel.visibility = if (mShowBottomBar) View.VISIBLE else View.GONE
        et_name.setText(mCreateParam.title)
        et_name.isCursorVisible = false
        refreshSelectView()
        setEditable(mEditEnable)
    }

    private fun showInput(editText: EditText) {
        mInputMethodManager = mInputMethodManager
                ?: (activity?.getSystemService(Context.INPUT_METHOD_SERVICE) as? InputMethodManager)
        mInputMethodManager?.showSoftInput(editText, -1)
    }

    private fun hideInput(editText: EditText) {
        mInputMethodManager?.hideSoftInputFromWindow(editText.windowToken, 0)
    }

    private fun refreshSelectView() {
        val period = mCreateParam.period
        val uidLimit = mCreateParam.uidLimit
        val periodF = String.format(resources.getString(R.string.trans_share_period_format), period)
        val uidLimitF = String.format(resources.getString(R.string.trans_share_cnt_format), uidLimit)
        val text1 = TextTools.highlightText(periodF, resources.getColor(R.color.trans_text_color_151515),
                true, true, period.toString())
        val text2 = TextTools.highlightText(uidLimitF, resources.getColor(R.color.trans_text_color_151515),
                true, true, uidLimit.toString())
        tv_wheel?.apply {
            text = ""
            append(text1)
            append(", ")
            append(text2)
        }
        setLastConfig(period, uidLimit)

        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_EXPIRED_CLICK,
                period.toString())
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_UIDLIMIT_CLICK,
                uidLimit.toString())
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_TITLE_CLICK,
                mCreateParam.title)
    }

    private fun setLastConfig(expired: Int, uidLimit: Int) {
        var eValue = expired
        var uValue = uidLimit
        if (eValue !in 1..30) {
            eValue = 7
        }
        if (uValue !in 1..500) {
            uValue = 60
        }
        PersonalConfig.getInstance().putInt(getConfigKey(TransAssistantConfigKey.TRANS_SHARE_EXPIRED_SUFFIX), eValue)
        PersonalConfig.getInstance().putInt(getConfigKey(TransAssistantConfigKey.TRANS_SHARE_UID_SUFFIX), uValue)
    }

    private fun getConfigKey(type: String): String {
        return "$type${AccountUtils.getInstance().uid}"
    }

    private fun initData() {
        ViewModelProviders.of(this).get(TransShareViewModel::class.java).apply {
            mTransAssManager = (activity as? BaseActivity)?.getService(BaseActivity.TRANS_ASSISTANT_SERVICE)
            mCurrentTaskResponse = mTransTaskResponse
            mCreatePCodeLiveData.observe(this@TransShareBottomSheet, Observer {
                NetDiskLog.d(TAG, "CreatePCodeLiveData has receiver")
                it?.getOrNull()?.let { pair ->
                    mViewModel?.processPCodeSuccess(activity!!, pair.first, pair.second)
                }
            })
            mCreateTaskLiveData.observe(this@TransShareBottomSheet, Observer {
                NetDiskLog.d(TAG, "CreateTaskLiveData has receiver")
                it?.getOrNull()?.let { pair ->
                    mViewModel?.processCreateTaskSuccess(activity!!, pair.first, pair.second)
                }
            })
            mShareErrorLiveData.observe(this@TransShareBottomSheet, Observer {
                NetDiskLog.d(TAG, "ShareErrorLiveData has receiver")
                it?.getFailureOrNull()?.let { fail ->
                    if (fail.errorCode == ErrorCode.TRANS_CREATE_SAME_NAME) {
                        showSameNameNotice(activity!!, object : DialogCtrListener {
                            override fun onOkBtnClick() {
                                processSameNameTask(activity!!, CreateTransTaskBean.DUP_NORMAL, fail.opCode, fail.data)
                            }

                            override fun onCancelBtnClick() {
                                processSameNameTask(activity!!, CreateTransTaskBean.DUP_OVERRIDE, fail.opCode, fail.data)
                            }
                        })
                    } else {
                        val resId = if (fail.errorType == ErrorType.NETWORK_ERROR) {
                            R.string.network_exception_message
                        } else {
                            R.string.trans_create_share_fail
                        }
                        ToastHelper.showToast(resId)
                    }
                } ?: ToastHelper.showToast(R.string.trans_create_share_fail)
            })
            mShareResultLiveData.observe(this@TransShareBottomSheet, Observer {
                it?.getOrNull()?.let {
                    mShareListener?.invoke(mCreateParam, true, mViewModel?.mCurrentTaskResponse)
                    setEditable(false)
                }
            })
        }.also {
            mViewModel = it
        }
    }

    private fun initItemListener() {
        iv_close.setOnClickListener {
            et_name?.let {
                hideInput(it)
            }
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_CLOSE_CLICK)
            dismiss()
        }
        tv_share_finish.setOnClickListener {
            et_name?.let {
                hideInput(it)
            }
            activity?.let {
                it.startActivity(TransListRecordActivity.getIntent(it))
                dismiss()
            }
        }
        tv_wx.setOnClickListener {
            handShare(CreateTransTaskBean.CHANNEL_WX, CreateTransTaskBean.OP_MINI)
        }
        tv_qq.setOnClickListener {
            handShare(CreateTransTaskBean.CHANNEL_QQ, CreateTransTaskBean.OP_PCODE)
        }
        tv_friend.setOnClickListener {
            handShare(CreateTransTaskBean.CHANNEL_FRIEND, CreateTransTaskBean.OP_FRIEND)
        }
        tv_sina.setOnClickListener {
            handShare(CreateTransTaskBean.CHANNEL_SINA, CreateTransTaskBean.OP_PCODE)
        }
        tv_wx_quan.setOnClickListener {
            handShare(CreateTransTaskBean.CHANNEL_WX_QUAN, CreateTransTaskBean.OP_PCODE)
        }
        tv_qrcode.setOnClickListener {
            handShare(CreateTransTaskBean.CHANNEL_QRCODE, CreateTransTaskBean.OP_QRCODE)
        }
        tv_pcode.setOnClickListener {
            handShare(CreateTransTaskBean.CHANNEL_PCODE, CreateTransTaskBean.OP_PCODE)
        }
        tv_qq_zone.setOnClickListener {
            handShare(CreateTransTaskBean.CHANNEL_QZONE, CreateTransTaskBean.OP_PCODE)
        }
        rl_wheel.setOnClickListener {
            showWheelChoose()
        }
    }

    private fun handShare(channel: Int, opCode: Int) {
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_CHANNEL_CLICK,
                channel.toString())
        activity?.let {
            mCreateParam.opCode = opCode
            mCreateParam.channel = channel
            mViewModel?.handleShare(it, channel, mCreateParam)
        }
    }

    private fun showWheelChoose() {
        if (!mEditEnable) return
        val menuDialog: BottomSheetDialog
        BottomSheetDialog(context!!, R.style.NetDisk_BottomSheetTrans).apply {
            setCancelable(false)
            setCanceledOnTouchOutside(false)
        }.also {
            menuDialog = it
        }
        LayoutInflater.from(context).inflate(R.layout.layout_trans_share_wheel, null).apply {
            val wvDate = findViewById<WheelView>(R.id.wv_date)
            val wvCnt = findViewById<WheelView>(R.id.wv_cnt)
            val tvCancel = findViewById<TextView>(R.id.tv_cancel)
            val tvSure = findViewById<TextView>(R.id.tv_sure)
            val periodAdapter = NumericStepWheelAdapter(context, 1, 30).apply {
                itemResource = R.layout.item_trans_share_wheel
                wvDate.isCyclic = false
                wvDate.viewAdapter = this
            }
            val uidAdapter = NumericStepWheelAdapter(context, 1, 500, 20, -1).apply {
                itemResource = R.layout.item_trans_share_wheel
                wvCnt.isCyclic = false
                wvCnt.viewAdapter = this
            }
            wvDate.currentItem = periodAdapter.getSelectIndex(mCreateParam.period)
            wvCnt.currentItem = uidAdapter.getSelectIndex(mCreateParam.uidLimit)
            tvCancel.setOnClickListener {
                menuDialog.dismiss()
            }
            tvSure.setOnClickListener {
                menuDialog.dismiss()
                mCreateParam.period = periodAdapter.getSelectValue(wvDate.currentItem)
                mCreateParam.uidLimit = uidAdapter.getSelectValue(wvCnt.currentItem)
                refreshSelectView()
            }
        }.also {
            menuDialog.setContentView(it)
        }
        menuDialog.show()
    }

    private fun setEditable(enable: Boolean) {
        mEditEnable = enable
        et_name.isEnabled = enable
        tv_wheel.isEnabled = enable
        rl_wheel.isEnabled = enable
        if (enable) {
            tv_name.visibility = View.GONE
            et_name.visibility = View.VISIBLE
            iv_icon.visibility = View.VISIBLE
            iv_arrow.visibility = View.VISIBLE
        } else {
            tv_name.text = mCreateParam.title
            tv_name.visibility = View.VISIBLE
            et_name.visibility = View.GONE
            iv_icon.visibility = View.GONE
            iv_arrow.visibility = View.INVISIBLE
        }
        // 特殊处理强制隐藏的逻辑
        if (!enable && mShowBottomBar) {
            tv_share_finish.visibility = View.VISIBLE
        } else {
            tv_share_finish.visibility = View.INVISIBLE
        }
    }
}