package com.baidu.netdisk.ui.xpan.transmission.viewmodel

import android.app.Activity
import android.app.Application
import android.arch.lifecycle.MutableLiveData
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.text.TextUtils
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.base.storage.config.ConfigSmartDevice
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.cloudfile.constant.FileManagerExtras
import com.baidu.netdisk.cloudfile.service.CloudFileServiceHelper
import com.baidu.netdisk.cloudfile.storage.db.CloudFileContract
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.main.caller.FSConstantApiCompManager
import com.baidu.netdisk.ui.widget.NewVersionDialog
import com.baidu.netdisk.ui.xpan.extension.getBDuss
import com.baidu.netdisk.ui.xpan.extension.getUid
import com.baidu.netdisk.ui.xpan.extension.model.CustomResult
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.FailedViewManager
import com.baidu.netdisk.util.receiver.ResultView
import com.baidu.netdisk.xpan.ITransAssistant
import com.baidu.netdisk.xpan.io.ErrorCode
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransReceiverListDetailBean
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransReceiverListDetailResponse
import com.google.gson.Gson
import com.google.gson.annotations.SerializedName

class TransListDetailViewModel(application: Application) : BaseFileOperateViewModel(application) {

    companion object {
        const val TAG = "TransListDetailViewModel"
    }

    var transManager: ITransAssistant? = null

    val transListLiveData = MutableLiveData<CustomResult<TransReceiverListDetailResponse>>()
    val transCopyLiveData = MutableLiveData<CustomResult<Int>>()
    val transErrorLiveData = MutableLiveData<CustomResult<Any>>()

    private val transListResultReceiver = TransListResultReceiver(this, Handler(), null)
    private lateinit var transCopyResultReceiver: CopyResultReceiver
    private lateinit var transCopyResultView: CopyResultView
    private var limit: Int = 50

    init {
        ConfigSmartDevice(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.SMART_DEVICE)).also {
            limit = it.customizeFileLimit
        }
    }

    fun getTransListDetail(start: Int, uuid: String) {
        NetDiskLog.d(TAG, "getTransListDetail:$start uuid:$uuid")
        transManager?.getTransDetail(getApplication(), transListResultReceiver, getBDuss(), getUid(), start, limit, uuid)
    }

    override fun copyTransTask(dest: String, copyFiles: List<TransReceiverListDetailBean>) {
        getWeakActivity()?.let {
            showVMLoading(it, R.string.trans_saveing)
            if (!this::transCopyResultReceiver.isInitialized) {
                transCopyResultView = CopyResultView(it)
                transCopyResultReceiver = CopyResultReceiver(this, Handler(), transCopyResultView)
            }
        } ?: return
        val uuid = copyFiles[0].uuid
        val list = copyFiles.filter {
            TextUtils.isDigitsOnly(it.senderId) && TextUtils.isDigitsOnly(it.uuid)
        }.map {
            val id = Long.runCatching {
                it.fsId.toLong()
            }
            val sendId = Long.runCatching {
                it.senderId.toLong()
            }
            Item(it.tid, id.getOrNull() ?: -1, sendId.getOrNull() ?: -1)
        }
        val jsonArray = Gson().toJson(list)
        NetDiskLog.d(TAG, "copyTransTask:$jsonArray uuid:$uuid")
        transManager?.copyTransTask(getApplication(), transCopyResultReceiver, getBDuss(), getUid(), uuid,
                dest, jsonArray)
    }


    private inner class TransListResultReceiver(reference: TransListDetailViewModel, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<TransListDetailViewModel>(reference, handler, resultView) {
        override fun onSuccess(reference: TransListDetailViewModel, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            resultData?.getParcelable<TransReceiverListDetailResponse>(BaseExtras.RESULT)?.takeIf {
                it.tasks != null
            }?.apply {
                transListLiveData.postValue(CustomResult.success(this))
            } ?: transErrorLiveData.postValue(CustomResult.failure(null, -1))
        }

        override fun onFailed(reference: TransListDetailViewModel, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            transErrorLiveData.postValue(CustomResult.failure(errType, errno))
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    private inner class CopyResultView(activity: Activity) : ResultView(activity) {

        override fun initFailedManager(activity: Activity, failedViewManager: FailedViewManager) {
            failedViewManager.setDefaultErrorView(FailedViewManager.FailedViewBean())
            val builder = NewVersionDialog.Builder(activity)
            builder.setSingleConfirmText(R.string.filemanager_i_know)
            failedViewManager.put(ErrorCode.TRANS_COPY_FILE_OVERFLOW, FailedViewManager.FailedViewBean(builder))
        }

        override fun closeAllView() {
            dismissVMLoading()
        }

        override fun getFailedMessage(type: ErrorType, errno: Int, resultData: Bundle,
                                      activity: Activity): String? {
            if (type == ErrorType.NETWORK_ERROR) {
                return activity.getString(R.string.network_exception_message)
            }
            if (errno != 0) {
                return when (errno) {
                    ErrorCode.TRANS_COPY_FILE_OVERFLOW -> {
                        val numLimit = resultData.getInt(BaseExtras.RESULT_FAILED, DEFAULT_LIMIT)
                        return activity.getString(R.string.trans_copy_failed_exceed_maxnum, numLimit)
                    }
                    ErrorCode.TRANS_FILE_NOT_EXIST -> activity.getString(R.string.trans_copy_file_not_exist)
                    ErrorCode.TRANS_HAS_RUNNING_TASK -> activity.getString(R.string.filemanager_has_task_running)
                    else -> activity.getString(R.string.trans_save_fail)
                }
            }
            return activity.getString(R.string.trans_save_fail)
        }

        override fun showSuccessView(resultData: Bundle?) {
            super.showSuccessView(resultData)
            closeAllView()
            ToastHelper.showToast(R.string.trans_save_success)
        }

        override fun showOperatingView(resultData: Bundle?) {
            super.showOperatingView(resultData)
            closeAllView()
            resultData?.takeUnless {
                it.isEmpty
            }?.let {
                // 获取正在处理的action，然后判断是正在处于何种状态，对于新建文件夹的操作来说，区分强刷的操作
                val action = resultData.getString(BaseExtras.RESULT)
                if (CloudFileServiceHelper.isRefreshAction(action)) {
                    ToastHelper.showToast(R.string.is_refreshing_try_later)
                }
            }
        }
    }

    private inner class CopyResultReceiver(reference: TransListDetailViewModel, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<TransListDetailViewModel>(reference, handler, resultView) {

        override fun onFailed(reference: TransListDetailViewModel, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            dismissVMLoading()
            transCopyLiveData.postValue(CustomResult.failure(errType, errno))
            getWeakActivity()?.takeIf {
                it is BaseActivity
            }?.let {
                return if (!processNoSpaceError(it as BaseActivity, errno)) {
                    super.onFailed(reference, errType, errno, resultData)
                } else {
                    !super.onFailed(reference, errType, errno, resultData)
                }
            }
            return super.onFailed(reference, errType, errno, resultData)
        }

        override fun onSuccess(reference: TransListDetailViewModel, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            transCopyLiveData.postValue(CustomResult.success(BaseStatus.SUCCESS))
        }

        override fun onOperating(reference: TransListDetailViewModel, resultData: Bundle?) {
            super.onOperating(reference, resultData)
            transCopyLiveData.postValue(CustomResult.success(BaseStatus.OPERATING))
            if (resultData == null || resultData.isEmpty) { // 开始异步任务的轮寻操作
                reference.getWeakActivity()?.apply {
                    val intent = Intent(this, FSConstantApiCompManager.getFileManagerProgressActivity())
                    intent.putExtra(FileManagerExtras.EXTRA_FILE_MANAGER_TASK_TYPE,
                            CloudFileContract.FileManagerTasks.TRANS_COPY)
                    startActivity(intent)
                }
            }
        }
    }

    private data class Item(
            @SerializedName("tid")
            val tid: String?,
            @SerializedName("fsid")
            val fsid: Long,
            @SerializedName("sender_id")
            val sendId: Long)

}