package com.baidu.netdisk.wap.offlinepackage

import android.support.annotation.WorkerThread
import android.text.TextUtils
import com.baidu.neidisk.model.TradeManifestItem
import com.baidu.neidisk.parser.TradePackageParser
import com.baidu.netdisk.base.storage.config.ConfigH5Package
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.dynamic.DynamicPluginConstant
import com.baidu.netdisk.dynamic.DynamicPluginConstant.PluginTypes
import com.baidu.netdisk.dynamic.base.storage.db.DynamicPluginProviderHelper
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import java.io.File

/**
 * 离线包数据管理器，供业务侧使用
 * todo 后续将config和manifestItemMap拆分成两个类分别在BaiduNetdisk和FastView模块中
 *
 * author guochao02
 * email guochao02@baidu.com
 * created 2019/11/4 11:18
 */
class OfflinePackageManager {

    private val H5_PACKAGE_TYPES = intArrayOf(PluginTypes.OFFLINE_H5_PACKAGE)

    private var configH5Package: ConfigH5Package = ConfigH5Package(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.H5_OFFLINE_SETTING_CONFIG))

    companion object {
        @JvmField
        val manifestItemMap = HashMap<String, OfflinePackage>()
        const val TAG = "OfflinePackageManager"
        const val BOOK_HOME_PAGE = 1
        const val DOC_DETAIL_PAGE = 2
        const val PIC_DETAIL_PAGE = 3
        const val TRADE_STORE_PAGE = 4

        const val VISIT_STATE_NATIVE = 0
        const val VISIT_STATE_OFFLINE = 1
        const val VISIT_STATE_ONLINE = 2


        // 交易平台数据字段
        const val ALL = 0 // 所有商品
        const val IMAGE = 3 // 图片
        const val AUDIO = 2 // 音频
        const val VIDEO = 1 // 视频
        const val DOCUMENT = 4 // 文件
        const val ATLAS = 6  // 图集
        const val JOURNAL = 8  // 图集
        const val UNIVERSAL = 45 // 通用
    }

    /**
     * 初始化离线包信息，供业务端获取使用
     */
    @WorkerThread
    fun initManifestItemMap() {
        val h5PackageList = DynamicPluginProviderHelper().getLocalPluginByTypes(H5_PACKAGE_TYPES)
        if (h5PackageList != null) {
            synchronized(manifestItemMap) {
                for (dynamicPlugin in h5PackageList) {
                    NetDiskLog.d(TAG, "H5Offline dynamicPlugin:$dynamicPlugin.id state:${dynamicPlugin.state}")
                    if (dynamicPlugin.state == DynamicPluginConstant.PluginStatus.ENABLE) {
                        val path = dynamicPlugin.executor!!.getInstallPath(dynamicPlugin)
                        NetDiskLog.d(TAG, "H5Offline path:$path")
                        val offlinePackage = OfflinePackage(dynamicPlugin.id)
                        offlinePackage.rootPath = dynamicPlugin.executor!!.getInstallPath(dynamicPlugin)
                        offlinePackage.tradeManifestItem = TradePackageParser().parse(File(path + File.separator + "manifest.json"))
                        offlinePackage.version = dynamicPlugin.version
                        manifestItemMap[dynamicPlugin.id] = offlinePackage
                        NetDiskLog.d(TAG, "H5Offline path:${manifestItemMap[dynamicPlugin.id]} size:${manifestItemMap.size}")
                    }
                }
            }
        }
    }

    /**
     * 获取跳转页面的url
     */
    fun getForwardUrl(pageType: Int): String? {
        return when (pageType) {
            BOOK_HOME_PAGE -> getBusinessUrl(configH5Package.pagesmap.bookhome)
            DOC_DETAIL_PAGE -> getBusinessUrl(configH5Package.pagesmap.docdetail)
            PIC_DETAIL_PAGE -> getBusinessUrl(configH5Package.pagesmap.picdetail)
            TRADE_STORE_PAGE -> getBusinessUrl(configH5Package.pagesmap.shoppage)
            else -> null
        }
    }

    /**
     * 获取页面的展现形式 native/h5/offline
     */
    fun getVisitState(pageType: Int): Int {
        return when (pageType) {
            BOOK_HOME_PAGE -> configH5Package.pagesmap.bookhome.viewstate
            DOC_DETAIL_PAGE -> configH5Package.pagesmap.docdetail.viewstate
            PIC_DETAIL_PAGE -> configH5Package.pagesmap.picdetail.viewstate
            TRADE_STORE_PAGE -> configH5Package.pagesmap.shoppage.viewstate
            else -> VISIT_STATE_NATIVE
        }
    }

    /**
     * 获取页面对应的离线包id
     */
    fun getPackageId(pageType: Int): String {
        return when (pageType) {
            BOOK_HOME_PAGE -> configH5Package.pagesmap.bookhome.pkgid
            DOC_DETAIL_PAGE -> configH5Package.pagesmap.docdetail.pkgid
            PIC_DETAIL_PAGE -> configH5Package.pagesmap.picdetail.pkgid
            TRADE_STORE_PAGE -> configH5Package.pagesmap.shoppage.pkgid
            else -> "-1"
        }
    }

    /**
     * 从配置文件中获取对应业务的访问地址
     */
    private fun getBusinessUrl(pagesmap: ConfigH5Package.DetailItem): String {
        var host = configH5Package.pagehost
        if (!TextUtils.isEmpty(pagesmap.host)) {
            host = pagesmap.host
        }
        return host + pagesmap.path
    }

    data class OfflinePackage(val id: String) {
        var rootPath: String = ""
        var tradeManifestItem: TradeManifestItem? = null
        var version: Int = 0
    }
}